/*********************************************************************
 Copyright (C) 2012-2014 by appSolutions a/s ("AS") -                    
 www.app-solutions.com and other contributors as listed below.  
 All Rights Reserved.                                       

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software. 

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
   File        : scm/custom/rtb/Utils/TempTableHelper.cls
   Purpose     : Generic methods for working with temp-table  
   Syntax      : 
                 DEFINE VARIABLE oTemptableHelper AS scm.custom.rtb.Utils.TempTableHelper NO-UNDO.
                 
                 oTemptableHelper = NEW scm.custom.rtb.Utils.TempTableHelper().
                    
   Description : 
   Created     : Thu Jul 03 15:10:40 CEST 2014
   Notes       : 
 ----------------------------------------------------------------------*/

USING Progress.Lang.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS scm.custom.rtb.Utils.TempTableHelper: 

    DEFINE PUBLIC PROPERTY DebugOn AS LOGICAL INITIAL FALSE NO-UNDO 
        GET.
        SET. 

    /*------------------------------------------------------------------------------
         Purpose: Generic method to convert from a LONGCHAR with JSON to a Temp-Table and return the handle. 
         Notes:
        ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE ConvertJsonToTtHandle( 
        INPUT plcJsondata AS LONGCHAR ):
        
        DEFINE VARIABLE hTT         AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cTargetType AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cReadMode   AS CHARACTER NO-UNDO.
         
        ASSIGN    
            cTargetType = "LONGCHAR"   
            cReadMode   = "EMPTY". 
        
        IF plcJsondata NE ? THEN 
        DO:
            CREATE TEMP-TABLE hTT. 
            hTT:READ-JSON(cTargetType, plcJsondata, cReadMode). 
        END.

        RETURN hTT.
        
        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err. 
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Generic method to convert a Temp-Table handle to a LONGCHAR with JSON
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR ConvertTtHandleToJson( 
        INPUT phTT AS HANDLE):
        
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.
        DEFINE VARIABLE cTargetType  AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lFormatted   AS LOGICAL   NO-UNDO.
         
        ASSIGN    
            cTargetType = "LONGCHAR"   
            lFormatted  = TRUE. 
        
        IF VALID-HANDLE(phTT) THEN 
        DO:
            IF (phTT:TYPE NE "DATASET" AND phTT:HAS-RECORDS) OR 
                (phTT:TYPE = "DATASET" AND (phTT:GET-BUFFER-HANDLE("Db"):TABLE-HANDLE:HAS-RECORDS)) THEN                
                phTT:WRITE-JSON (cTargetType, lcJsonResult, lFormatted).
        END.
        ELSE
            lcJsonResult = "":U.  

        RETURN lcJsonresult.
        
        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err. 
        END CATCH.

    END METHOD.
END CLASS.