/***********************************************************************
* Copyright (C) 2000,2012 by Progress Software Corporation. All rights *
* reserved. Prior versions of this work may contain portions         *
* contributed by participants of Possenet.                           *
*                                                                    *
*********************************************************************/
/*----------------------------------------------------------------------------

File                  : _getobject.p

Description           : This is the ADE API to do the call to open a Dynamic Object.

Input Parameters:
  phWindow            : Window in which to display the dialog box.
  gcProductModule     : Initial Product Module Code.
  glOpenInAppBuilder  : Open in Appbuilder
  pcTitle             : Title for Dialog Window to be launched

Output Parameters:
  gcFileName          : The filename selected.
  pressedOK           : TRUE if user successfully choose an object file name.

Author: Pieter J. Meyer
        Thomas Hansen

Date Created: September 2002
Date Created: August 2004

----------------------------------------------------------------------------*/
{adecomm/oeideservice.i}
DEFINE INPUT  PARAMETER phWindow           AS HANDLE      NO-UNDO.
DEFINE INPUT  PARAMETER gcProductModule    AS CHARACTER   NO-UNDO.
DEFINE INPUT  PARAMETER glOpenInAppBuilder AS LOGICAL     NO-UNDO.
DEFINE INPUT  PARAMETER pcTitle            AS CHARACTER   NO-UNDO.
DEFINE OUTPUT PARAMETER gcFileName         AS CHARACTER   NO-UNDO.
DEFINE OUTPUT PARAMETER pressedOK          AS LOGICAL     NO-UNDO.

DEFINE VARIABLE hRTBDeskHandle             AS HANDLE    NO-UNDO.
DEFINE VARIABLE cRTBAction                 AS CHARACTER NO-UNDO.

DEFINE VARIABLE cProduct                   AS CHARACTER NO-UNDO INITIAL "UIB":U.
DEFINE VARIABLE cAction                    AS CHARACTER NO-UNDO INITIAL "OPEN":U.
DEFINE VARIABLE cMode                      AS CHARACTER NO-UNDO.

{src/adm2/globals.i}

PUBLISH "evRtbGetDeskHandle":U (OUTPUT hRTBDeskHandle). 

IF VALID-HANDLE(hRTBDeskHandle)THEN 
DO:
    PUBLISH "evRtbAdeGetFile":U (
        INPUT  phWindow ,
        INPUT  cProduct,
        INPUT  cAction,
        INPUT  pcTitle,
        INPUT  cMode,
        INPUT-OUTPUT gcFileName,
        OUTPUT cRTBAction ).

    /* Process the action coming from RTB */
    CASE cRTBAction:
        WHEN "Cancel" OR 
        WHEN "" THEN 
            DO:
                ASSIGN 
                    pressedOK = NO.
                RETURN.
            END.
            
        WHEN "Go" THEN 
            DO:
                ASSIGN 
                    pressedOK = YES.      
                RETURN.
            END.
            
        OTHERWISE 
        DO: /* nothing */ 
        END.
    END CASE.
END.   /* can-do... */        

/* If we get this far, then "Open Via OS" has been selected - and we use the standard Open Object Dialog */
IF SEARCH("adeuib/_opendialog.w":U) <> ?
    OR SEARCH("adeuib/_opendialog.r":U) <> ?
    THEN
DO:  
    IF OEIDEIsRunning THEN 
        RUN adeuib/ide/_dialog_opendialog.p 
            (phWindow,
            gcProductModule,
            glOpenInAppBuilder,
            pcTitle,
            OUTPUT gcFileName,
            OUTPUT pressedOK).
    ELSE
        RUN adeuib/_opendialog.w(phWindow,
            gcProductModule,
            glOpenInAppBuilder,
            pcTitle,
            OUTPUT gcFileName,
            OUTPUT pressedOK).

END.                           
ELSE 
DO:
    ASSIGN
        pressedOK = NO.
    MESSAGE "No open object dialog is available."
        VIEW-AS ALERT-BOX BUTTONS OK.
    RETURN.

END.

RETURN.

