/**********************************************************************
* Copyright (C) 2016-present by nodeable limited, and                 *
* other contributors as listed below.  All Rights Reserved.           *
*                                                                     *
*  Software is distributed on an "AS IS", WITHOUT WARRANTY OF ANY     *
*   KIND, either express or implied.                                  *
*                                                                     *
*  Contributors:                                                      *
*  appSolutions, www.appsolutions.com                                 *
**********************************************************************/
/*
  Desc:
    Roundtable Event Handler for Plug-in
  Parameters:
    INPUT pcParam - Delimited list of event details
      ex: "|changeWorkspace|<rowid>|<wspace-id>,<etc>"

  NOTE:
  This custom verison of the event handler for the
  Windows plugin client extends the event handler with custom
  procedures or classes to handle the extual event handler
  extensions.

  For this to work, the files in the following directories
  from the customization must be available in the PROPATH of the
  client at startup:
  scm/custom/rtb
  scm/custom/rtb/utils

  These directories and files may need to be placed in the DLC/gui directory
  for the AVM in each RTB Project to be able to pick up the event handler extension classes.

*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING scm.custom.rtb.* FROM PROPATH.
DEFINE INPUT PARAMETER pcParam AS CHARACTER  NO-UNDO.

DEFINE VARIABLE cEvent         AS CHARACTER NO-UNDO.
DEFINE VARIABLE cContext       AS CHARACTER NO-UNDO.
DEFINE VARIABLE cContext2      AS CHARACTER NO-UNDO.
DEFINE VARIABLE cOther         AS CHARACTER NO-UNDO.
DEFINE VARIABLE cDelim         AS CHARACTER NO-UNDO.
DEFINE VARIABLE lOk            AS LOGICAL   NO-UNDO INITIAL YES.

DEFINE VARIABLE cErrorMessage  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cReturnMessage AS CHARACTER NO-UNDO.
DEFINE VARIABLE hLoopHandle    AS HANDLE    NO-UNDO.
DEFINE VARIABLE hRtbEvent      AS HANDLE    NO-UNDO.
DEFINE VARIABLE hPreRtbEvent   AS HANDLE    NO-UNDO.
DEFINE VARIABLE hXMLConfMan    AS HANDLE    NO-UNDO.
DEFINE VARIABLE hRTBDeskHandle AS HANDLE    NO-UNDO.

ASSIGN
    cDelim  = SUBSTRING(pcParam, 1, 1) /* delimiter is first char */
    pcParam = SUBSTRING(pcParam, 2).

IF NUM-ENTRIES(pcParam, cDelim) = 4 THEN
    ASSIGN
        cEvent    = ENTRY(1, pcParam, cDelim)
        cContext  = ENTRY(2, pcParam, cDelim)
        cContext2 = ENTRY(3, pcParam, cDelim)
        cOther    = ENTRY(4, pcParam, cDelim).
ELSE
    ASSIGN
        cEvent   = ENTRY(1, pcParam, cDelim)
        cContext = ENTRY(2, pcParam, cDelim)
        cOther   = ENTRY(3, pcParam, cDelim).

/*
 Uncomment for debugging..
*/
/*
   MESSAGE
     "Event:" cEvent SKIP
     "Context:" cContext SKIP
     "Other Data:" cOther SKIP 
     "Delimeter " cDelim SKIP 
     pcParam
     VIEW-AS ALERT-BOX QUESTION 
     BUTTONS OK-CANCEL
     TITLE "_idertb_events.p"
     /*UPDATE lOk */.
*/
/*   IF NOT lOk THEN  */
/*     RETURN "Error".*/
/*   ELSE             */
/*     RETURN "OK".   */


ASSIGN 
    cErrorMessage = "":U.

/*  NOTE FOR RTB DYNAMICS INTEGRATION: 
    The order of the events below is critical */
/* If we are not connected to ICFDB, then we don't want the integration code to kick in */
IF NOT CONNECTED("ICFDB":U) THEN 
    RETURN.

/* First walk the widget tree and get the handles for event 
   procedures that may already be running*/
ASSIGN 
    hLoopHandle = SESSION:FIRST-PROCEDURE.

DO WHILE VALID-HANDLE(hLoopHandle):
    IF NOT VALID-HANDLE(hRtbEvent) AND 
        (hLoopHandle:FILE-NAME    = "rtb/prc/ryrtbevntp.p":U OR    
        hLoopHandle:PRIVATE-DATA = "ryrtbevntp.p":U) THEN 
        ASSIGN hRtbEvent = hLoopHandle.

    IF NOT VALID-HANDLE(hPreRtbEvent) AND 
        (hLoopHandle:FILE-NAME    = "rtb/prc/rtbpreevent.p":U OR    
        hLoopHandle:PRIVATE-DATA = "rtbpreevent.p":U) THEN
        ASSIGN hPreRtbEvent = hLoopHandle.

    IF NOT VALID-HANDLE(hXMLConfMan) AND
        (R-INDEX(hLoopHandle:FILE-NAME,"afxmlcfgp.p":U) > 0 OR 
        R-INDEX(hLoopHandle:FILE-NAME,"afxmlcfgp.r":U) > 0) THEN
        ASSIGN hXMLConfMan = hLoopHandle.

    IF VALID-HANDLE(hRtbEvent) AND 
        VALID-HANDLE(hPreRtbEvent) AND 
        VALID-HANDLE(hXMLConfMan)   THEN
        ASSIGN hLoopHandle = ?.
    ELSE
        ASSIGN hLoopHandle = hLoopHandle:NEXT-SIBLING.
END.

/***********************************************************************************/
/* Pre-validation : Any code in this section is not bound to the Repository at all */
/***********************************************************************************/
IF NOT VALID-HANDLE(hPreRtbEvent) THEN 
    RUN rtb/prc/rtbpreevent.p PERSISTENT SET hPreRtbEvent NO-ERROR.

IF VALID-HANDLE(hPreRtbEvent) THEN
    RUN processPreEvent IN hPreRtbEvent
        (INPUT cEvent,
        INPUT  cContext,
        INPUT  cOther,
        INPUT  cContext2,
        OUTPUT cReturnMessage).

IF cReturnMessage NE "":U THEN 
DO:
    IF cReturnMessage = "Cancelled":U THEN 
        ASSIGN lOk = FALSE.
    ELSE 
        ASSIGN lOk = TRUE.
    RETURN.
END.

/*****************************/
    /* Standard Integration code */
/*****************************/
IF cEvent NE "logout" THEN 
DO:
    IF NOT VALID-HANDLE(hRtbEvent) THEN 
    DO:
        IF SEARCH("rtb/prc/ryrtbevntp.p":U) NE ? OR 
            SEARCH("rtb/prc/ryrtbevntp.r":U) NE ? THEN
            RUN rtb/prc/ryrtbevntp.p PERSISTENT SET hRtbEvent.         
    END.
END.

IF cEvent = "logout":U THEN
DO:        
    IF VALID-HANDLE(hXMLConfMan) THEN
        APPLY "CLOSE":U TO hXMLConfMan.    

    IF VALID-HANDLE(hRtbEvent) THEN
        RUN killPlip IN hRtbEvent.

    IF VALID-HANDLE(hPreRtbEvent) THEN    
        RUN killPlip IN hPreRtbEvent.

    ASSIGN 
        hXMLConfMan  = ?
        hRtbEvent    = ? 
        hPreRtbEvent = ?.        
END.

IF cEvent = "init":U THEN
DO:
    IF VALID-HANDLE(hRtbEvent) THEN
        RUN scmCheckDCUWorkspaceLock IN hRtbEvent
            (INPUT cOther,
            OUTPUT cErrorMessage).

    IF cErrorMessage NE "":U THEN
    DO:
        IF VALID-HANDLE(hRtbEvent) THEN
            RUN killPlip IN hRtbEvent.

        RETURN "ERROR".
    END.
    /*  Start the Dynamics environment */
    IF VALID-HANDLE(hRtbEvent) THEN
        RUN icfstart.p NO-ERROR.
END.

IF VALID-HANDLE(hRtbEvent) THEN
    RUN processEvent IN hRtbEvent (INPUT cEvent,
        INPUT  cContext,
        INPUT  cOther,
        INPUT  cContext2,
        OUTPUT cErrorMessage).


IF cErrorMessage NE "":U THEN
    MESSAGE
        "RTB Error: " + cErrorMessage
        VIEW-AS ALERT-BOX 
        INFORMATION 
        BUTTONS OK.

IF cErrorMessage NE "":U THEN 
    ASSIGN lOk = FALSE.
ELSE
    ASSIGN lOk = TRUE.

IF NOT lOk THEN
    RETURN "Error".
ELSE
    RETURN "OK".
