/*********************************************************************
* Copyright (C) 2006-2018 by Roundtable Software Inc. ("Roundtable"),*
*  All Rights Reserved.                                              *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
* nodeable limited (www.nodeable.io)                                 *
*********************************************************************/
/*------------------------------------------------------------------------
  File        : rtb_events.p
  Description : Roundtable published event handler
  Notes       :

    The procedure is run persistently in the Roundtable session and
    receives the published event as documented below. To intercept an
    invent, modify 'evRtbUserEvent' as needed to respond to the desired
    published event.


 Roundtable Published Events and Parameters
   
 Client Events:
   These events are available in a client-server session or from rtb_events_cl.p 
   on the client when in an AppServer session.  
    
  pcEvent    = {&CHANGE-TASK}
  pcOther    = Task Number
  
  pcEvent    = {&CHANGE-WORKSPACE-BEFORE}
  pcContext  = Desktop handle string value
  pcOther    = From-Workspace, To-Workspace
  plOk       = False will cancel
  
  pcEvent    = {&CHANGE-WORKSPACE}
  pcContext  = Desktop handle string value
  pcOther    = Workspace ID
  
  pcEvent    = {&MOVE-TO-WEB-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Pmod, Module
  plOk       = False will cancel 
  
  pcEvent    = {&MOVE-TO-WEB}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name
  
  pcEvent    = {&NEW-TASK-ROW}
  pcContext  = RowObject buffer handle string value
  pcOther    = Workspace ID
  
  pcEvent    = {&ROUNDTABLE-STARTUP}
  pcContext  = Tabletop procedure handle string value
  pcOther    = Tabletop window handle string value
  
  pcEvent    = {&ROUNDTABLE-SHUTDOWN}
                                                                                                          
  pcEvent    = {&SESSION-STARTUP}
  pcContext  = Session ID string value
  plOk       = False will cancel
  
  pcEvent    = {&SESSION-SHUTDOWN}
  pcContext  = Session ID string value
    
 Server Events:
   These events are available in a client-server session from this procedure
   on the AppServer when running in an AppServer configuration.  

  pcEvent    = {&ADD-OBJECT-BEFORE}
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  plOk       = False will cancel
  
  pcEvent    = {&ADD-OBJECT}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  
  pcEvent    = {&ADD-OBJECT-BACKUP}
  pcContext  = rtb_backup ROWID string value
  pcOther    = User, Backup-type, Reference GUID
  pcContext2 = rtb_backup.backup-guid
  
  pcEvent    = {&ADD-OBJECT-LAB-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  plOk       = False with cancel
    
  pcEvent    = {&ADD-OBJECT-LAB}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
    
  pcEvent    = {&ASSIGN-OBJECT-BEFORE}
  pcOther    = Object Name, Object Type, Product Module, Version, Workspace ID
  plOk       = False will cancel
  
  pcEvent    = {&ASSIGN-OBJECT}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid, rtb_hist.hist-guid
  
  pcEvent    = {&BUILD-IMPORT-LIST-BEFORE}
  pcOther    = Target Workspace
  plOk       = False will cancel
      
  pcEvent    = {&BUILD-IMPORT-LIST}
  pcOther    = Target Workspace
  
  pcEvent    = {&BUILD-SCHEMA-LIST-BEFORE }
  pcOther    = Database Name
  plOk       = False will cancel
      
  pcEvent    = {&BUILD-SCHEMA-LIST}
  pcOther    = Database Name
  
  pcEvent    = {&CHANGE-OBJECT-SHARE-STATUS-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Old Status, New Status, WorkspacePath, Task Path
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel
  
  pcEvent    = {&CHANGE-OBJECT-SHARE-STATUS}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Old Status, New Status, WorkspacePath, Task Path
  pcContext2 = rtb_object.object-guid
  
  pcEvent    = {&CHANGE-PROPATH-BEFORE}
  pcContext  = rtb_object ROWID string value            
  pcOther    = "BEFORE-COMPILE" or "AFTER-COMPILE"            
              
  pcEvent    = {&CHANGE-PROPATH}                                                                         
  pcContext  = rtb_object ROWID string value
  pcOther    = "BEFORE-COMPILE" or "AFTER-COMPILE"
  
  pcEvent    = {&CHECKIN-OBJECT-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel 
  
  pcEvent    = {&CHECKIN-OBJECT}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid, rtb_hist.hist-guid
  
  pcEvent    = {&CHECKOUT-OBJECT-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID, Level ("version","revision","patch")
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel
  
  pcEvent    = {&CHECKOUT-OBJECT}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
                                                                                                          
  pcEvent    = {&COMPILE-OBJECT-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel
  
  pcEvent    = {&COMPILE-OBJECT}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
                                                                                                          
  pcEvent    = {&COMPILE-RECORD-CREATE}
  pcContext  = Buffer handle of compile record string value
                                                                                                          
  pcEvent    = {&COMPLETE-TASK-BEFORE}
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
  plOk       = False will cancel
  
  pcEvent    = {&COMPLETE-TASK}
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
  
  pcEvent    = {&CREATE-DEPLOYMENT-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Site Code
  plOk       = False will cancel
  
  pcEvent    = {&CREATE-DEPLOYMENT}
  pcContext  = rtb_moddef ROWID string value
  pcOther    = Site Code
  pcContext2 = rtb_deploy.deploy-guid
                                                                                                          
  pcEvent    = {&CREATE-MODULE-DEF-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Module
  plOk       = False will cancel
  
  pcEvent    = {&CREATE-MODULE-DEF}
  pcContext  = rtb_moddef ROWID string value
  pcOther    = Module
  pcContext2 = rtb_moddef.moddef-guid
                                                                                                          
  pcEvent    = {&CREATE-OBJECT-VARIANT-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Current Product Module, New Produce Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel
  
  pcEvent    = {&CREATE-OBJECT-VARIANT}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
                                                                                                          
  pcEvent    = {&CREATE-PRODUCT-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Product Id
  plOk       = False will cancel 
  
  pcEvent    = {&CREATE-PRODUCT}
  pcContext  = rtb_product ROWID string value
  pcOther    = Product Id
  pcContext2 = rtb_product.product-guid
                                                                                                          
  pcEvent    = {&CREATE-PRODUCT-MODULE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Product Module
  plOk       = False will cancel 
  
  pcEvent    = {&CREATE-PRODUCT-MODULE}
  pcContext  = rtb_pmod ROWID string value
  pcOther    = Product Module
  pcContext2 = rtb_pmod.pmod-guid
                                                                                                          
  pcEvent    = {&CREATE-RELEASE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id
  plOk       = False will cancel
  
  pcEvent    = {&CREATE-RELEASE}
  pcContext  = rtb_release ROWID string value
  pcOther    = Workspace ID, Release Number
  pcContext2 = rtb_release.release-guid 
                                                                                                          
  pcEvent    = {&CREATE-SITE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace ID, License Type, Site Code
  plOk       = False will cancel
  
  pcEvent    = {&CREATE-SITE}
  pcContext  = rtb_site ROWID string value 
  pcOther    = Workspace ID, License Type, Site Code
  pcContext2 = rtb_site.site-guid
                                                                                                          
  pcEvent    = {&CREATE-SUBTYPE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value  
  pcOther    = Subtype 
  plOk       = False will cancel 
  
  pcEvent    = {&CREATE-SUBTYPE}
  pcContext  = rtb_subtype ROWID string value
  pcOther    = Subtype
  pcContext2 = rtb_subtype.subtype-guid
                                                                                                          
  pcEvent    = {&CREATE-TASK-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = CHR(1)-delimited list of column/value pairs
  plOk       = False will cancel
  
  pcEvent    = {&CREATE-TASK}
  pcContext  = rtb_task ROWID string value
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
                                                                                                          
  pcEvent    = {&CREATE-WORKSPACE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id
  plOk       = False will cancel 
  
  pcEvent    = {&CREATE-WORKSPACE}
  pcContext  = rtb_wspace ROWID string value
  pcOther    = Workspace Id 
  pcContext2 = rtb_wspace.wspace-guid
                                                                                                          
  pcEvent    = {&CREATE-WORKSPACE-SOURCE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id, Product Id 
  plOk       = False will cancel 
  
  pcEvent    = {&CREATE-WORKSPACE-SOURCE}
  pcContext  = rtb_wssrc ROWID string value
  pcOther    = Workspace Id, Product Id 
  pcContext2 = rtb_wssrc.wssrc-guid
  
  pcEvent    = {&DELETE-DEPLOYMENT-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Site Code
  pcContext2 = rtb_deploy.deploy-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-DEPLOYMENT}
  pcContext  = rtb_deploy ROWID string value
  pcOther    = Site Code
  pcContext2 = rtb_deploy.deploy-guid
                                                                                                          
  pcEvent    = {&DELETE-MODULE-DEF-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Module
  pcContext2 = rtb_moddef.moddef-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-MODULE-DEF}
  pcContext  = rtb_moddef ROWID string value
  pcOther    = Module
  pcContext2 = rtb_moddef.moddef-guid
                                                                                                          
  pcEvent    = {&DELETE-OBJECT-BEFORE}
  pcContext  = rtb_object ROWID string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-OBJECT}
  pcContext  = WIP version of the deleted object
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid, rtb_hist.hist-guid 
                                                                                                          
  pcEvent    = {&DELETE-PRODUCT-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Product Id
  pcContext2 = rtb_product.product-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-PRODUCT}
  pcContext  = rtb_product ROWID string value
  pcOther    = Product Id
  pcContext2 = rtb_product.product-guid
                                                                                                          
  pcEvent    = {&DELETE-PRODUCT-MODULE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Product Module
  pcContext2 = rtb_pmod.pmod-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-PRODUCT-MODULE}
  pcContext  = rtb_pmod ROWID string value 
  pcOther    = Product Module
  pcContext2 = rtb_pmod.pmod-guid

  pcEvent    = {&DELETE-RELEASE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace
  pcContext2 = rtb_release.release-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-RELEASE}
  pcContext  = rtb_release ROWID string value
  pcOther    = Workspace
  pcContext2 = rtb_release.subtype-guid
  
  pcEvent    = {&DELETE-SITE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace ID, License Type, Site Code
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-SITE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace ID, License Type, Site Code
  pcContext2 = rtb_site.site-guid
                                                                                                            
  pcEvent    = {&DELETE-SUBTYPE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Subtype
  pcContext2 = rtb_subtype.subtype-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-SUBTYPE}
  pcContext  = rtb_subtype ROWID string value
  pcOther    = Subtype
  pcContext2 = rtb_subtype.subtype-guid
    
  pcEvent    = {&DELETE-TASK-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-TASK}
  pcContext  = rtb_task ROWID string value
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
                                                                                                          
  pcEvent    = {&DELETE-WORKSPACE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id
  pcContext2 = rtb_wspace.wspace-guid
  plOk       = False will cancel
  
  pcEvent    = {&DELETE-WORKSPACE}
  pcContext  = rtb_wspace ROWID string value
  pcOther    = Workspace Id
  pcContext2 = rtb_wspace.wspace-guid
  
  pcEvent    = {&DELETE-WORKSPACE-SOURCE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id, Product Id 
  pcContext2 = rtb_wssrc.wssrc-guid
  plOk       = False will cancel 
  
  pcEvent    = {&DELETE-WORKSPACE-SOURCE}
  pcContext  = rtb_wssrc ROWID string value
  pcOther    = Workspace Id, Product Id 
  pcContext2 = rtb_wssrc.wssrc-guid
                                                                                                          
  pcEvent    = {&LOAD-PARTNER-BEFORE}
  pcContext  = Workspace ID
  pcOther    = Workspace Path
  plOk       = False will cancel
  
  pcEvent    = {&LOAD-PARTNER}
  pcContext  = rtb_wspace ROWID string value
                                                                                                          
  pcEvent    = {&LOGIN-BEFORE}
  pcOther    = User Id
  plOk       = False will cancel 
  
  pcEvent    = {&LOGIN}
  pcContext  = rtb_session ROWID string value
  pcOther    = User Id
  pcContext2 = rtb_session.session-guid
                                                                                                          
  pcEvent    = {&LOGOUT-BEFORE}
  pcContext  = rtb_session ROWID string value
  pcOther    = Session Id, Programmer
  pcContext2 = rtb_session.session-guid
  plOk       = False will cancel
  
  pcEvent    = {&LOGOUT}
  pcOther    = Session Id
                                                                                                          
  pcEvent    = {&MAKE-DEPLOYMENT-BEFORE}
  pcContext  = rtb_site ROWID string value
  pcOther    = rtb_deploy ROWID string value
  pcContext2 = rtb_deploy.deploy-guid
  plOk       = False will cancel
  
  pcEvent    = {&MAKE-DEPLOYMENT}
  pcContext  = rtb_site ROWID string value
  pcOther    = rtb_deploy ROWID string value
  pcContext2 = rtb_deploy.deploy-guid
  
  pcEvent    = {&MODIFY-DEPLOYMENT-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Site Code
  pcContext2 = rtb_deploy.deploy-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-DEPLOYMENT}
  pcContext  = rtb_deploy ROWID string value
  pcOther    = Site Code
  pcContext2 = rtb_deploy.deploy-guid
                                                                                                        
  pcEvent    = {&MODIFY-MODULE-DEF-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Module
  pcContext2 = rtb_moddef.moddef-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-MODULE-DEF}
  pcContext  = rtb_moddef ROWID string value
  pcOther    = Module
  pcContext2 = rtb_moddef.moddef-guid
                                                                                                          
  pcEvent    = {&MODIFY-PRODUCT-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Product Id
  pcContext2 = rtb_product.product-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-PRODUCT}
  pcContext  = rtb_product ROWID string value
  pcOther    = Product Id
  pcContext2 = rtb_product.product-guid
                                                                                                          
  pcEvent    = {&MODIFY-PRODUCT-MODULE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Product Module
  pcContext2 = rtb_pmod.pmod-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-PRODUCT-MODULE}
  pcContext  = rtb_pmod ROWID string value
  pcOther    = Product Module
  pcContext2 = rtb_pmod.pmod-guid
                                                                                                          
  pcEvent    = {&MODIFY-RELEASE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace
  pcContext2 = rtb_release.release-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-RELEASE}
  pcContext  = rtb_release ROWID string value
  pcOther    = Workspace
  pcContext2 = rtb_release.release-guid
  
  pcEvent    = {&MODIFY-SITE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace ID, License Type, Site Code
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-SITE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace ID, License Type, Site Code
  pcContext2 = rtb_site.site-guid

  
  pcEvent    = {&MODIFY-SUBTYPE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Subtype
  pcContext2 = rtb_subtype.subtype-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-SUBTYPE}
  pcContext  = rtb_subtype ROWID string value
  pcOther    = Subtype
  pcContext2 = rtb_subtype.subtype-guid
                                                                                                          
  pcEvent    = {&MODIFY-TASK-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Task Number 
  pcContext2 = rtb_task.task-guid
  plOk       = False will cancel 
  
  pcEvent    = {&MODIFY-TASK}
  pcContext  = rtb_task ROWID string value
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
                                                                                                          
  pcEvent    = {&MODIFY-WORKSPACE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id
  pcContext2 = rtb_wspace.wspace-guid
  plOk       = False will cancel
  
  pcEvent    = {&MODIFY-WORKSPACE}
  pcContext  = rtb_wspace ROWID string value
  pcOther    = Workspace Id
  pcContext2 = rtb_wspace.wspace-guid
                                                                                                          
  pcEvent    = {&MODIFY-WORKSPACE-SOURCE-BEFORE}
  pcContext  = RowObjUpd buffer handle string value
  pcOther    = Workspace Id, Product Id 
  pcContext2 = rtb_wssrc.wssrc-guid
  plOk       = False will cancel 
  
  pcEvent    = {&MODIFY-WORKSPACE-SOURCE}
  pcContext  = rtb_wssrc ROWID string value
  pcOther    = Workspace Id, Product Id 
  pcContext2 = rtb_wssrc.wssrc-guid
                                                                                                          
  pcEvent    = {&PROCESS-IMPORT-BEFORE}
  pcContext  = Workspace ID
  plOk       = False will cancel 
  
  pcEvent    = {&PROCESS-IMPORT}
  pcContext  = Workspace ID
               
  pcEvent    = {&REOPEN-TASK-BEFORE}
  pcContext  = rtb_task ROWID string value
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
  plOk       = False will cancel
  
  pcEvent    = {&REOPEN-TASK}
  pcContext  = rtb_task ROWID string value 
  pcOther    = Task Number
  pcContext2 = rtb_task.task-guid
                                                                                                          
  pcEvent    = {&SMART-COMPILE-BEFORE}
  pcContext  = Temp-table handle of compile specs string value
  plOk       = False will cancel
  
  pcEvent    = {&UPDATE-SCHEMA-BEFORE}
  pcContext  = Workspace ID
  plOk       = False will cancel
  
  pcEvent    = {&UPDATE-SCHEMA}
  pcContext  = Workspace ID
                                                                                                          
  pcEvent    = {&XREF-OBJECT-BEFORE}
  pcContext  = Buffer-handle of compile record string value
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
  plOk       = False will cancel
  
  pcEvent    = {&XREF-OBJECT}
  pcContext  = Buffer-handle of compile record string value                                                                                           
  pcOther    = Object Name, Object Type, Product Module, Workspace ID
  pcContext2 = rtb_object.object-guid
 
 ----------------------------------------------------------------------*/

{rtb/i/rtb_events.i}

SUBSCRIBE TO {&RTB-USER-EVENT} ANYWHERE.

PROCEDURE {&RTB-USER-EVENT}:
/*------------------------------------------------------------------------------
  Purpose:     Roundtable Event Handler
  Parameters: INPUT
                pcEvent    - Event preprocessor
                pcContext  - Context string (rowident, buffer-handle, etc)
                pcOther    - Additional data
                pcContext2 - GUID value of entity related to event.
              OUTPUT
               plOk        - FALSE will cancel *BEFORE* events.
  Notes:       Common procedure for all events.
              RETURN a value from this procedure to return a message
               to the caller.
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pevent   AS CHARACTER.
  DEFINE INPUT  PARAMETER Pcontext AS CHARACTER.
  DEFINE INPUT  PARAMETER Pother   AS CHARACTER.
  DEFINE INPUT  PARAMETER Pcontext2 AS CHARACTER.
  DEFINE OUTPUT PARAMETER Pok      AS LOGICAL INITIAL YES.

   DEFINE VARIABLE cErrorMessage       AS CHARACTER  NO-UNDO INITIAL "".
    DEFINE VARIABLE cReturnMessage AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hLoopHandle    AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hRtbEvent      AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hPreRtbEvent   AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hXMLConfMan    AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hRTBDeskHandle AS HANDLE    NO-UNDO.
    DEFINE VARIABLE lDebugOn       AS LOGICAL   NO-UNDO INITIAL FALSE.

    /* Uncomment for debugging */

    /*
    MESSAGE
      "Event:   " Pevent SKIP
      "Context: " Pcontext SKIP
      "Other:   " Pother
  
      VIEW-AS ALERT-BOX
      BUTTONS OK-CANCEL
      TITLE "Roundtable Event"
      UPDATE Pok.
    */

    ASSIGN
        cErrorMessage = "":U.

    IF NOT CONNECTED("ICFDB":U) THEN
        RETURN.

    /* First walk the widget tree and get the handles */
    /* See if already running first */
    ASSIGN
        hLoopHandle = SESSION:FIRST-PROCEDURE.

    DO WHILE VALID-HANDLE(hLoopHandle):

        IF NOT VALID-HANDLE(hRtbEvent) AND
            ( hLoopHandle:FILE-NAME    = "rtb/prc/ryrtbevntp.p":U OR
            hLoopHandle:PRIVATE-DATA = "ryrtbevntp.p":U) THEN
            ASSIGN hRtbEvent = hLoopHandle.

        IF NOT VALID-HANDLE(hPreRtbEvent) AND
            ( hLoopHandle:FILE-NAME    = "rtb/prc/rtbpreevent.p":U OR
            hLoopHandle:PRIVATE-DATA = "rtbpreevent.p":U) THEN
            ASSIGN hPreRtbEvent = hLoopHandle.

        IF NOT VALID-HANDLE(hXMLConfMan) AND
            (R-INDEX(hLoopHandle:FILE-NAME,"afxmlcfgp.p":U) > 0 OR
            R-INDEX(hLoopHandle:FILE-NAME,"afxmlcfgp.r":U) > 0) THEN
            ASSIGN hXMLConfMan = hLoopHandle.

        IF  VALID-HANDLE(hRtbEvent) AND
            VALID-HANDLE(hPreRtbEvent) AND
            VALID-HANDLE(hXMLConfMan) THEN
            ASSIGN hLoopHandle = ?.
        ELSE
            ASSIGN hLoopHandle = hLoopHandle:NEXT-SIBLING.
    END.

    /***********************************************************************************/
    /* Pre-validation : Any code in this section is not bound to the Repository at all */
    /***********************************************************************************/
    IF NOT VALID-HANDLE(hPreRtbEvent) THEN
        RUN rtb/prc/rtbpreevent.p PERSISTENT SET hPreRtbEvent NO-ERROR.

    IF VALID-HANDLE(hPreRtbEvent) THEN
        RUN processPreEvent IN hPreRtbEvent
            (INPUT pEvent,
            INPUT pContext,
            INPUT pOther,
            INPUT pContext2,
            OUTPUT cReturnMessage).
    IF cReturnMessage NE "":U THEN
    DO:
        IF cReturnMessage = "Cancelled":U THEN
            ASSIGN pOk = FALSE.
        ELSE
            ASSIGN pOk = TRUE.
        RETURN.
    END.

    /*****************/
    /* Standard Code */
    /*****************/

    IF  pEvent  NE "changeWorkspaceBefore" THEN
    DO:
        IF NOT VALID-HANDLE(hRtbEvent) THEN
        DO:
            IF SEARCH("rtb/prc/ryrtbevntp.p":U) NE ? OR
                SEARCH("rtb/prc/ryrtbevntp.r":U) NE ? THEN
                RUN rtb/prc/ryrtbevntp.p PERSISTENT SET hRtbEvent.

            /* Set the debug status */
            PUBLISH "evRtbCustSetDebug" (INPUT lDebugOn).
        END.
    END.

    IF  pEvent   = "changeWorkspaceBefore":U AND
        ENTRY(1,pOther) NE "":U THEN /* only if previous workspace valid - kill running plips */
    DO:
        IF VALID-HANDLE(hXMLConfMan) THEN
            APPLY "CLOSE":U TO hXMLConfMan.

        IF VALID-HANDLE(hRtbEvent) THEN
            RUN killPlip IN hRtbEvent.

        IF VALID-HANDLE(hPreRtbEvent) THEN
            RUN killPlip IN hPreRtbEvent.

        ASSIGN
            hXMLConfMan  = ?
            hRtbEvent    = ?
            hPreRtbEvent = ?.

        /* Remove the Progress Dynamics menu from the tabletop */
        PUBLISH "evRtbGetDeskHandle":U (OUTPUT hRTBDeskHandle).
        RUN rtb/prc/afrtbmenup.p (INPUT hRTBDeskHandle).
    END.

    IF pEvent = "changeWorkspace":U THEN
    DO:
        FIND FIRST rtb.rtb_wspace WHERE rtb_wspace.wspace-id = pOther NO-LOCK NO-ERROR.
        IF AVAILABLE rtb_wspace THEN
            IF INDEX(rtb_wspace.locked-by, "DCU":U) > 0 THEN
            DO:
                ASSIGN
                    cErrorMessage = "The Dynamics environment has not been started. The workspace is locked : " + "~n" +
                                    "'" + rtb_wspace.locked-by + "'" + "~n":U + "~n":U +
                                   "Access to the workspace is limited as the integration hooks between RTB" + "~n":U +
                                   "and Dynamics are not enabled. " + "~n":U + "~n":U +
                                   "Wait until the DCU upgrade has been completed and re-select the workspace, " + "~n":U +
                                   "or contact your system administrator for further information.".

                IF VALID-HANDLE(hRtbEvent) THEN
                    RUN killPlip IN hRtbEvent.

                RETURN.
            END.

        /* Start the Dynamics environment  */
        IF VALID-HANDLE(hRtbEvent) THEN
            RUN icfstart.p /* NO-ERROR */.

        /* Add the Progress Dynamics menu to the tabletop */
        /* Set Grtb-proc-handle  */
        PUBLISH "evRtbGetDeskHandle":U (OUTPUT hRTBDeskHandle).
        RUN rtb/prc/afrtbmenup.p (INPUT hRTBDeskHandle) NO-ERROR.

    END.

    IF VALID-HANDLE(hRtbEvent) THEN
        RUN processEvent IN hRtbEvent
            (INPUT pEvent,
            INPUT  pContext,
            INPUT  pOther,
            INPUT  pContext2,
            OUTPUT cErrorMessage).

    IF cErrorMessage NE "":U THEN
        MESSAGE
            "RTB Error: " + cErrorMessage
            VIEW-AS ALERT-BOX
            INFORMATION
            BUTTONS OK.

    IF cErrorMessage NE "":U THEN
        ASSIGN pOk = FALSE.
    ELSE
        ASSIGN pOk = TRUE.

    RETURN.

END PROCEDURE.
